import * as z from 'zod';
import z__default from 'zod';
import * as react from 'react';
import * as zod_to_json_schema from 'zod-to-json-schema';
import OpenAI from 'openai';

type TActionBuilderConstructorData<TRegistry extends TAnyRegistryData, TId extends string> = Pick<TActionData<TRegistry, TId, any, any, any, any>, "metadata" | "id" | "registryData">;
type TActionBuilderData = TActionBuilderConstructorData<TAnyRegistryData, string>;
declare class ActionBuilder<TLocalActionData extends TActionBuilderData> {
    actionData: TLocalActionData;
    constructor(input: TLocalActionData);
    /**
     * The Zod input schema for the action. This is used to validate the arguments passed to the action.
     *
     * You must include a `.describe()` call on each field to provide a description of the input. This description will be used to generate the prompts and details page for your action.
     *
     * @example
     * The following input schema is an example for a `send-email` action.
     * ```
     * z.object({
     *  subject: z.string().desribe("The subject of the email"),
     *  body: z.string().describe("The body of the email"),
     *  to: z.array(z.object({
     *   email: z.string().describe("The email of the recipient"),
     * })).describe("The recipients of the email"),
     * ```
     */
    setInputSchema<T extends TActionInput>(input: T): ActionBuilderWithInput<TLocalActionData & {
        inputSchema: T;
    }>;
}

type TActionBuilderWithFunctionData<TAuthActionData extends TActionDataWithAuth> = TAuthActionData["authConfig"] extends infer U ? U extends TActionAuth<TAuthActionData["registryData"], TTokenCustomData> ? TAuthActionData & Pick<TActionData<TAuthActionData["registryData"], TAuthActionData["id"], TAuthActionData["inputSchema"], TAuthActionData["outputSchema"], U, TAuthActionData["submissionSchema"]>, "actionFunction" | "exampleInput"> : never : never;
type TActionDataWithFunction = TActionBuilderWithFunctionData<TActionDataWithAuth>;
declare class ActionBuilderWithFunction<TLocalActionData extends TActionDataWithFunction> {
    actionData: TLocalActionData;
    constructor({ actionData }: {
        actionData: TLocalActionData;
    });
    setExampleInput(exampleInput: z__default.input<TLocalActionData["inputSchema"]>): this;
    getId(): TLocalActionData["id"];
    getExampleInput(): TLocalActionData["exampleInput"];
    getNamespace(): TLocalActionData["registryData"]["namespace"];
    getMetadata(): TLocalActionData["metadata"];
    getAuthConfig(): TLocalActionData["authConfig"];
    getInputSchema(): TLocalActionData["inputSchema"];
    getSubmissionSchema(): TLocalActionData["submissionSchema"];
    getActionFunction(): TLocalActionData["actionFunction"];
    getOutputSchema(): TLocalActionData["outputSchema"];
    getInputJSONSchema(): zod_to_json_schema.JsonSchema7Type & {
        $schema?: string | undefined;
        definitions?: {
            [key: string]: zod_to_json_schema.JsonSchema7Type;
        } | undefined;
    };
    getOutputJSONSchema(): zod_to_json_schema.JsonSchema7Type & {
        $schema?: string | undefined;
        definitions?: {
            [key: string]: zod_to_json_schema.JsonSchema7Type;
        } | undefined;
    };
    getComponent(): TLocalActionData["component"];
    getRegistryData(): TLocalActionData["registryData"];
}

type TActionBuilderWithAuthData<TOutputActionData extends TActionDataWithOutput, TAuth extends TActionAuth<TOutputActionData["registryData"], TTokenCustomData>> = TOutputActionData & {
    authConfig: TAuth;
};
type TActionDataWithAuth = TActionBuilderWithAuthData<TActionDataWithOutput, TAnyActionAuth>;
declare class ActionBuilderWithAuth<TLocalActionData extends TActionDataWithAuth> {
    actionData: TLocalActionData;
    constructor({ actionData }: {
        actionData: TLocalActionData;
    });
    /**
     * The function that gets called when the action is invoked. This function should return a promise that resolves to the output of the action.
     *
     * @param input The input to the action function. This is the input that the user provides when invoking the action.
     * @param auth The authentication information for the action. This is the authentication information that the user provides when invoking the action.
     *
     * @returns A promise that resolves to the output of the action.
     *
     * @example
     * The following action function is an example for a `send-email` action.
     * ```
     * async ({ input, auth }) => {
     *   const { subject, body, to } = input;
     *   const { accessToken } = auth;
     *   ...code to send email using the input...
     *   return;
     * }
     * ```
     *
     * @example
     * The following action function is an example for a `get-google-contact` action.
     * ```
     * async ({ input, auth }) => {
     *   const contact = ...code to get contact...
     *   return contact;
     * }
     * ```
     */
    setActionFunction(actionFunction: TActionFunction<TLocalActionData["registryData"], TLocalActionData["inputSchema"], TLocalActionData["outputSchema"], TLocalActionData["authConfig"], TLocalActionData["submissionSchema"]>): ActionBuilderWithFunction<TLocalActionData & {
        actionFunction: TActionFunction<TLocalActionData["registryData"], TLocalActionData["inputSchema"], TLocalActionData["outputSchema"], TLocalActionData["authConfig"], TLocalActionData["submissionSchema"]>;
        exampleInput: null;
    }>;
}

declare class ActionBuilderWithOAuthType<TLocalActionData extends TActionDataWithOutput> {
    actionData: TLocalActionData;
    constructor({ actionData }: {
        actionData: TLocalActionData;
    });
    setOAuthData: (data: TOAuthConfigWithInputMetadata<TLocalActionData["registryData"]>) => ActionBuilderWithAuth<TLocalActionData & {
        authConfig: {
            type: "OAuth";
            config: TOAuthConfigWithOutputMetadata<TLocalActionData["registryData"]>;
        };
    }>;
}

declare class ActionBuilderWithTokenType<TLocalActionData extends TActionDataWithOutput> {
    actionData: TLocalActionData;
    constructor({ actionData }: {
        actionData: TLocalActionData;
    });
    setTokenData: <T extends TTokenCustomData>(data: TTokenAuthConfigWithInputMetadata<T, TLocalActionData["registryData"]>) => ActionBuilderWithAuth<TLocalActionData & {
        authConfig: TTokenAuth<T, TLocalActionData["registryData"]>;
    }>;
}

type TNoAuth$1 = {
    type: typeof AuthType.NONE;
    config: undefined;
};
type TActionBuilderWithOutputData<TInputActionData extends TActionDataWithInput, TOutput extends TActionOutput, TSubmission extends TActionOnSubmit> = TInputActionData & Pick<TActionData<TInputActionData["registryData"], TInputActionData["id"], TInputActionData["inputSchema"], TOutput, any, TSubmission>, "outputSchema" | "component" | "submissionSchema">;
type TActionDataWithOutput = TActionBuilderWithOutputData<TActionDataWithInput, TActionOutput, TActionOnSubmit>;
declare class ActionBuilderWithOutput<TLocalActionData extends TActionDataWithOutput> {
    actionData: TLocalActionData;
    constructor({ actionData }: {
        actionData: TLocalActionData;
    });
    /**
     * The authentication type for the action. This is used to generate the authentication page
     * for the action and get users authenticated.
     */
    setAuthType(type: typeof AuthType.NONE): ActionBuilderWithAuth<TActionBuilderWithAuthData<TLocalActionData, TNoAuth$1>>;
    setAuthType(type: typeof AuthType.TOKEN): ActionBuilderWithTokenType<TLocalActionData>;
    setAuthType(type: typeof AuthType.OAUTH): ActionBuilderWithOAuthType<TLocalActionData>;
}

declare class ActionBuilderWithGet<TLocalActionData extends TActionDataWithInput> {
    actionData: TLocalActionData;
    constructor({ actionData }: {
        actionData: TLocalActionData;
    });
    /**
     * `JSON Data` that adheres to a specific Zod schema. This is typically the output for actions that `GET data`. If this is the case, you should specify the Zod schema for the output.
     */
    setOutputSchema<T extends z__default.ZodObject<any>>(output: T): ActionBuilderWithOutput<TLocalActionData & {
        submissionSchema: undefined;
        outputSchema: T;
        component: null;
    }>;
}

type TActionBuilderWithInputData<TBuilderData extends TActionBuilderData, TInput extends TActionInput> = TBuilderData & Pick<TActionData<any, any, TInput, any, any, any>, "inputSchema">;
type TActionDataWithInput = TActionBuilderWithInputData<TActionBuilderData, TActionInput>;
type TActionType = "GET" | "POST";
declare class ActionBuilderWithInput<TLocalActionData extends TActionDataWithInput> {
    actionData: TLocalActionData;
    constructor({ actionData }: {
        actionData: TLocalActionData;
    });
    /**
     * Spark has two options for actions:
     *
     * 1.`JSON Data` that adheres to a specific Zod schema. This is typically the output for actions that `GET data`. If this is the case, you should specify the Zod schema for the output.
     *
     * 2.`React Component` that asks for confirmation for the action. This is typically the output for actions that `POST data`. If this is the case, you should specify a React component that asks for confirmation for the action. The output will be `void`.
     */
    setActionType(type: "GET"): ActionBuilderWithGet<TLocalActionData>;
    setActionType(type: "POST"): ActionBuilderWithPost<TLocalActionData, undefined>;
}

type TActionComponentPropsData<TInput extends TActionInput, TOnSubmitValues extends TActionOnSubmit> = {
    /**
     * The input to the action function. You can use this input to prepopulate the form so all the user has to do is click submit.
     */
    input: z__default.output<TInput>;
    /**
     * Indicates whether the action function is currently running.
     */
    isLoading: boolean;
    /**
     * Indicates whether the action function has completed running and is successful.
     */
    isSuccess: boolean;
    /**
     * Indicates whether the action function has completed running and has failed.
     */
    isError: boolean;
    /**
     * The submit handler for the form. This is called when the user clicks the submit button.
     */
    onSubmit: (formValues: z__default.infer<TOnSubmitValues extends undefined ? TInput : TOnSubmitValues>) => Promise<void>;
};
type TActionComponentProps<TMetadata extends TActionMetadata, TInput extends TActionInput, TOnSubmitValues extends TActionOnSubmit> = {
    inputSchema: TInput;
    metadata: TMetadata extends z__default.AnyZodObject ? z__default.output<TMetadata> : undefined;
} & ({
    /**
     *  When the action is active, the user can interact with the component.
     */
    displayState: "active";
    data: TActionComponentPropsData<TInput, TOnSubmitValues>;
} | {
    /**
     * A placeholder component is used for things like the action details page, where the user can view the component but not interact with it.
     */
    displayState: "placeholder";
    data: undefined;
} | {
    /**
     * Indicates whether the `LLM is still streaming in the input to the action function`.
     * If this is the case, it is necessary to show a loading state of the form.
     * We recommend using Skeleton components instead of input fields to indicate that the form is loading.
     */
    displayState: "skeleton";
    data: undefined;
});
type TActionComponent<TMetadata extends TActionMetadata, TInput extends TActionInput, TOnSubmitValues extends TActionOnSubmit> = React.FC<TActionComponentProps<TMetadata, TInput, TOnSubmitValues>>;
type TPassThroughComponent<TMetadata extends TActionMetadata, TInput extends TActionInput, TOutput extends TActionOutput, TOnSubmitValues extends TActionOnSubmit> = TOutput extends z__default.ZodVoid ? TActionComponent<TMetadata, TInput, TOnSubmitValues> : null;
/**
 * For actions that do not return any data (i.e. POST actions), you need to specify a component to render in the chat after the action is invoked.
 * This component should be a form asking for confirmation to submit the input to the action function.
 */
declare class ActionBuilderWithPost<TLocalActionData extends TActionDataWithInput, TSubmission extends TActionOnSubmit> {
    actionData: TLocalActionData;
    submissionSchema: TSubmission;
    constructor({ actionData, submissionSchema, }: {
        actionData: TLocalActionData;
        submissionSchema: TSubmission;
    });
    setComponentSubmissionSchema<T extends TActionInput>(submissionSchema: T): ActionBuilderWithPost<TLocalActionData, T>;
    setOutputComponent(component: TLocalActionData["registryData"] extends infer TRegistry ? TRegistry extends TAnyRegistryData ? TLocalActionData["inputSchema"] extends infer TInput ? TRegistry["metadataSchema"] extends infer TMetadata ? TInput extends TActionInput ? TMetadata extends TActionMetadata ? React.FC<TActionComponentProps<TMetadata, TInput, TSubmission>> : never : never : never : never : never : never): ActionBuilderWithOutput<TLocalActionData & {
        submissionSchema: TSubmission;
        outputSchema: z__default.ZodVoid;
        component: TLocalActionData["registryData"] extends infer TRegistry ? TRegistry extends TAnyRegistryData ? TLocalActionData["inputSchema"] extends infer TInput ? TRegistry["metadataSchema"] extends infer TMetadata ? TInput extends TActionInput ? TMetadata extends TActionMetadata ? react.FC<TActionComponentProps<TMetadata, TInput, TSubmission>> : never : never : never : never : never : never;
    }>;
}

type TActionMetadata = z__default.ZodObject<any> | undefined;
type TActionFunctionExtras = z__default.ZodObject<any> | undefined;
type TActionInput = z__default.ZodObject<any>;
type TActionOutput = z__default.ZodObject<any> | z__default.ZodVoid;
type TActionOnSubmit = z__default.ZodObject<any> | undefined;
type TTokenAuthMetadata = z__default.ZodObject<any> | undefined;
type TOAuthMetadata = z__default.ZodObject<any> | undefined;
type TActionFunction<TRegistry extends TAnyRegistryData, TInput extends TActionInput, TOutput extends TActionOutput, TAuth extends TAnyActionAuth, TSubmission extends TActionOnSubmit> = (_: {
    input: z__default.output<TSubmission extends undefined ? TInput : TSubmission>;
    auth: TAuthArg<TAuth>;
    extras: TRegistry["actionFunctionExtrasSchema"] extends infer U ? U extends z__default.AnyZodObject ? z__default.infer<U> : undefined : undefined;
}) => Promise<z__default.infer<TOutput>>;
type TRegistryData<TNamespace extends string, TMetadata extends TActionMetadata, TExtras extends TActionFunctionExtras, TToken extends TTokenAuthMetadata, TOAuth extends TOAuthMetadata> = {
    /**
     * The namespace for the actions registry. Useful if you have multiple action registries.
     *
     * @example "ADE"
     */
    namespace: TNamespace;
    /**
     * The schema for the metadata of the action. This is used to validate the metadata passed to the action.
     *
     * If you don't have metadata, you can pass `undefined`.
     *
     * @example
     * ```typescript
     * z.object({
     *   title: z.string(),
     *   description: z.string(),
     * })
     * ```
     */
    metadataSchema?: TMetadata;
    /**
     * Sometimes you may want to pass extra data to each action function (on top of input and auth). This is useful for things like user data, local time zone, etc.
     *
     * If you don't have any extra data, you can pass `undefined`.
     *
     * @example
     * ```typescript
     * z.object({
     *  userData: z.object({
     *   email: z.string().email(),
     *   name: z.string(),
     *  })
     * })
     * ```
     */
    actionFunctionExtrasSchema?: TExtras;
    /**
     * The schema for the token auth metadata. Useful if you want to store some extra data with the token.
     *
     * ```typescript
     * z.object({
     *  tokenGenerationDocumentationURL: z.string().url(),
     * })
     */
    tokenAuthMetadataSchema?: TToken;
    /**
     * The schema for the OAuth metadata. Useful if you want to store some extra data with the OAuth token.
     *
     * ```typescript
     * z.object({
     *   tokenGenerationDocumentationURL: z.string().url(),
     * })
     */
    oAuthMetadataSchema?: TOAuth;
};
type TAnyRegistryData = TRegistryData<string, TActionMetadata, TActionFunctionExtras, TTokenAuthMetadata, TOAuthMetadata>;
type TActionData<TRegistry extends TAnyRegistryData, TId extends string, TInput extends TActionInput, TOutput extends TActionOutput, TAuth extends TAnyActionAuth, TSubmission extends TActionOnSubmit> = {
    registryData: TRegistry;
    id: TId;
    metadata: TRegistry["metadataSchema"] extends infer U ? U extends z__default.AnyZodObject ? z__default.output<U> : undefined : undefined;
    inputSchema: TInput;
    submissionSchema?: TSubmission;
    outputSchema: TOutput;
    authConfig: TAuth;
    actionFunction: TActionFunction<TRegistry, TInput, TOutput, TAuth, TSubmission>;
    component: TRegistry["metadataSchema"] extends infer U ? U extends TActionMetadata ? TPassThroughComponent<U, TInput, TOutput, TSubmission> : never : never;
    exampleInput: z__default.input<TInput> | null;
};

type ValuesOf<T> = T[keyof T];
type FilterObjectByValue<T, U> = {
    [K in keyof T]: T[K] extends U ? K : never;
};

/**
 * The type of authentication.
 */
declare const AuthType: {
    readonly NONE: "None";
    readonly TOKEN: "Token";
    readonly OAUTH: "OAuth";
};
type TAuthType = ValuesOf<typeof AuthType>;
type TAuthInputMetadata<TRegistry extends TAnyRegistryData, K extends "tokenAuthMetadataSchema" | "oAuthMetadataSchema"> = TRegistry[K] extends infer U ? U extends z__default.AnyZodObject ? z__default.input<U> : {} : {};
type TAuthOutputMetadata<TRegistry extends TAnyRegistryData, K extends "tokenAuthMetadataSchema" | "oAuthMetadataSchema"> = TRegistry[K] extends infer U ? U extends z__default.AnyZodObject ? z__default.output<U> : {} : {};
declare const OAuthConfigSchema: z__default.ZodUnion<[z__default.ZodObject<{
    scopes: z__default.ZodArray<z__default.ZodString, "many">;
    discoveryEndpoint: z__default.ZodString;
}, "strip", z__default.ZodTypeAny, {
    scopes: string[];
    discoveryEndpoint: string;
}, {
    scopes: string[];
    discoveryEndpoint: string;
}>, z__default.ZodObject<{
    scopes: z__default.ZodArray<z__default.ZodString, "many">;
    discoveryEndpoint: z__default.ZodUndefined;
    authorizationEndpoint: z__default.ZodString;
    tokenEndpoint: z__default.ZodString;
    refreshEndpoint: z__default.ZodOptional<z__default.ZodString>;
    revokeEndpoint: z__default.ZodOptional<z__default.ZodString>;
    codeChallengeMethod: z__default.ZodNullable<z__default.ZodLiteral<"S256">>;
}, "strip", z__default.ZodTypeAny, {
    scopes: string[];
    authorizationEndpoint: string;
    tokenEndpoint: string;
    codeChallengeMethod: "S256" | null;
    discoveryEndpoint?: undefined;
    refreshEndpoint?: string | undefined;
    revokeEndpoint?: string | undefined;
}, {
    scopes: string[];
    authorizationEndpoint: string;
    tokenEndpoint: string;
    codeChallengeMethod: "S256" | null;
    discoveryEndpoint?: undefined;
    refreshEndpoint?: string | undefined;
    revokeEndpoint?: string | undefined;
}>]>;
type TOAuthConfig = z__default.infer<typeof OAuthConfigSchema>;
type TOAuthConfigWithOutputMetadata<TRegistry extends TAnyRegistryData> = TOAuthConfig & TAuthOutputMetadata<TRegistry, "oAuthMetadataSchema">;
type TOAuthConfigWithInputMetadata<TRegistry extends TAnyRegistryData> = TOAuthConfig & TAuthInputMetadata<TRegistry, "oAuthMetadataSchema">;
type TTokenCustomData = z__default.ZodObject<any> | null;
/**
 * Configuration for authenticating with an access token that users must generate themselves.
 *
 * This is useful for resources that don't support OAuth, but do support API keys.
 */
type TTokenAuthConfig<TCustomData extends TTokenCustomData> = {
    /**
     * Optional schema for custom data to be stored in the user's account that is needed to use the action.
     *
     * @example
     * For the Canvas LTS action, we also need to collect the user's Canvas domain (e.g. `https://canvas.calpoly.edu`).
     * ```
     * z.object({
     *  canvasDomain: z.string().url().refine((url) => url.startsWith("https://"), {
     *   message: "Canvas domain must start with https://",
     *  }),
     * })
     * ```
     */
    customDataSchema: TCustomData;
    /**
     * An optional function to validate the access token and custom data upon submission.
     *
     * @example
     * For the Canvas LTS action, we need to validate the access token and custom data by making a request to the Canvas API.
     *
     * ```typescript
     * validateToken: async ({ auth }) => {
     *  const response = await fetch(`${auth.customData.canvasDomain}/api/v1/users/self`, {
     *   headers: {
     *    Authorization: `Bearer ${auth.accessToken}`,
     *   },
     *  })
     *
     *  return { isValid: response.ok }
     * }
     * ```
     */
    validateToken?: (_: {
        auth: {
            customData: TCustomData extends z__default.ZodObject<any> ? z__default.infer<TCustomData> : null;
            accessToken: string;
        };
    }) => Promise<{
        isValid: boolean;
    }>;
};
type TTokenAuthConfigWithInputMetadata<TCustomData extends TTokenCustomData, TRegistry extends TAnyRegistryData> = TTokenAuthConfig<TCustomData> & TAuthInputMetadata<TRegistry, "tokenAuthMetadataSchema">;
type TTokenAuthConfigWithOutputMetadata<TCustomData extends TTokenCustomData, TRegistry extends TAnyRegistryData> = TTokenAuthConfig<TCustomData> & TAuthOutputMetadata<TRegistry, "tokenAuthMetadataSchema">;
type TNoAuth = {
    type: typeof AuthType.NONE;
    config: undefined;
};
type TTokenAuth<TCustomData extends TTokenCustomData, TRegistry extends TAnyRegistryData> = {
    type: typeof AuthType.TOKEN;
    config: TTokenAuthConfigWithOutputMetadata<TCustomData, TRegistry>;
};
type TOAuth<TRegistry extends TAnyRegistryData> = {
    type: typeof AuthType.OAUTH;
    config: TOAuthConfigWithOutputMetadata<TRegistry>;
};
/**
 * The authentication configuration for an action.
 */
type TActionAuth<TRegistry extends TAnyRegistryData, TCustomData extends TTokenCustomData> = TNoAuth | TTokenAuth<TCustomData, TRegistry> | TOAuth<TRegistry>;
type TAnyActionAuth = TActionAuth<TAnyRegistryData, TTokenCustomData>;
/**
 * Generic type for getting the auth argument in the action input function.
 *
 * The auth argument will always have an `accessToken` property, but it may also have a `customData` property if the auth config has a custom data schema.
 */
type TAuthArg<TAuth extends TAnyActionAuth> = TAuth["type"] extends typeof AuthType.NONE ? never : TAuth["type"] extends typeof AuthType.OAUTH ? {
    accessToken: string;
} : TAuth["type"] extends typeof AuthType.TOKEN ? TAuth["config"] extends TTokenAuthConfigWithOutputMetadata<infer TCustomData, TAnyRegistryData> ? TCustomData extends TTokenCustomData ? {
    customData: TCustomData extends z__default.AnyZodObject ? z__default.output<TCustomData> : null;
    accessToken: string;
} : never : never : never;

type TAnyActionRegistry = Readonly<{
    [key: string]: ActionBuilderWithFunction<any>;
}>;
type TActionCallerInput<TRegistry extends TAnyActionRegistry, TId extends keyof TRegistry> = ValuesOf<{
    [K in TId]: {
        actionId: K;
        arguments: z__default.input<ReturnType<TRegistry[K]["getInputSchema"]>>;
        id?: string;
    };
}>;
type TCallerSuccessResult<TAction extends ActionBuilderWithFunction<any>> = {
    status: "success";
    data: z__default.output<ReturnType<TAction["getOutputSchema"]>>;
    actionId: ReturnType<TAction["getId"]>;
    inputId: string;
};
type TCallerErrorResult<TAction extends ActionBuilderWithFunction<any>> = {
    status: "error";
    message: string;
    cause?: Error;
    actionId: ReturnType<TAction["getId"]>;
    inputId: string;
};
type TActionRegistrySubset<TRegistry extends TAnyActionRegistry, TSubset extends (keyof TRegistry)[] | undefined = undefined> = TSubset extends (keyof TRegistry)[] ? TSubset[number] : keyof TRegistry;
type TCallerResults<TRegistry extends TAnyActionRegistry, TId extends keyof TRegistry> = ValuesOf<{
    [K in TId]: TCallerSuccessResult<TRegistry[K]> | TCallerErrorResult<TRegistry[K]>;
}>[];
type inferExtras<TActionData extends TActionDataWithFunction, T extends Readonly<{
    [K in TActionData["id"]]: ActionBuilderWithFunction<TActionData>;
}>> = ReturnType<ValuesOf<T>["getRegistryData"]> extends infer TRegistry ? TRegistry extends TAnyRegistryData ? TRegistry["actionFunctionExtrasSchema"] extends infer A ? A extends z__default.AnyZodObject ? {
    extras: z__default.input<A>;
} : {} : {} : {} : {};
type filterByAuthType<T extends TAnyActionRegistry, U extends TAuthType, TId extends keyof T, TSuccess> = {
    [K in TId as ReturnType<T[K]["getAuthConfig"]>["type"] extends U ? K : never]: TSuccess;
};
/**
 * A function that fetches the OAuth access token for an action.
 *
 * @param actionId The ID of the action to fetch the OAuth access token for.
 * @returns A promise that resolves to the OAuth access token as { accessToken: string }.
 *
 * @throws An error if the OAuth access token could not be fetched.
 *
 * @example
 * ```ts
 * async fetchOAuthAccessToken(actionId) {
 *    ...fetch the OAuth access token...
 *    return {
 *        accessToken: "the access token"
 *    }
 * }
 *
 */
type TFetchOAuthAccessToken<T> = (actionId: T) => Promise<{
    accessToken: string;
}>;
type inferOAuthFunction<TRegistry extends TAnyActionRegistry, TId extends keyof TRegistry> = filterByAuthType<TRegistry, "OAuth", TId, 1> extends infer A ? keyof A extends never ? {} : {
    fetchOAuthAccessToken: TFetchOAuthAccessToken<keyof A>;
} : {};
type TFetchTokenAuthData<T> = (actionId: T) => Promise<{
    accessToken: string;
    customData: z__default.AnyZodObject["_input"];
}>;
type inferTokenFunction<TRegistry extends TAnyActionRegistry, TId extends keyof TRegistry> = filterByAuthType<TRegistry, "Token", TId, 1> extends infer A ? keyof A extends never ? {} : {
    fetchTokenAuthData: TFetchTokenAuthData<keyof A>;
} : {};
type TFunctionCallingArgs<TRegistry extends TAnyActionRegistry, U extends (keyof TRegistry)[] | undefined = undefined> = {
    inArray?: U;
    runInParallel?: boolean;
} & inferExtras<TActionDataWithFunction, TRegistry> & inferOAuthFunction<TRegistry, TActionRegistrySubset<TRegistry, U>> & inferTokenFunction<TRegistry, TActionRegistrySubset<TRegistry, U>>;
type TActionCaller<TActionData extends TActionDataWithFunction, TRegistry extends Readonly<{
    [K in TActionData["id"]]: ActionBuilderWithFunction<TActionData>;
}>, U extends (keyof TRegistry)[] | undefined = undefined> = (inputs: TActionCallerInput<TRegistry, TActionRegistrySubset<TRegistry, U>>[]) => Promise<TCallerResults<TRegistry, TActionRegistrySubset<TRegistry, U>>>;
declare const setupActionCaller: <TActionData extends (TActionBuilderData & Pick<TActionData<any, any, TActionInput, any, any, any>, "inputSchema"> & Pick<TActionData<TAnyRegistryData, string, TActionInput, TActionOutput, any, TActionOnSubmit>, "submissionSchema" | "outputSchema" | "component"> & {
    authConfig: TAnyActionAuth;
} & Pick<TActionData<TAnyRegistryData, string, TActionInput, TActionOutput, {
    type: "None";
    config: undefined;
}, TActionOnSubmit>, "actionFunction" | "exampleInput">) | (TActionBuilderData & Pick<TActionData<any, any, TActionInput, any, any, any>, "inputSchema"> & Pick<TActionData<TAnyRegistryData, string, TActionInput, TActionOutput, any, TActionOnSubmit>, "submissionSchema" | "outputSchema" | "component"> & {
    authConfig: TAnyActionAuth;
} & Pick<TActionData<TAnyRegistryData, string, TActionInput, TActionOutput, TTokenAuth<TTokenCustomData, TAnyRegistryData>, TActionOnSubmit>, "actionFunction" | "exampleInput">) | (TActionBuilderData & Pick<TActionData<any, any, TActionInput, any, any, any>, "inputSchema"> & Pick<TActionData<TAnyRegistryData, string, TActionInput, TActionOutput, any, TActionOnSubmit>, "submissionSchema" | "outputSchema" | "component"> & {
    authConfig: TAnyActionAuth;
} & Pick<TActionData<TAnyRegistryData, string, TActionInput, TActionOutput, {
    type: "OAuth";
    config: TOAuthConfigWithOutputMetadata<TAnyRegistryData>;
}, TActionOnSubmit>, "actionFunction" | "exampleInput">), const TRegistry extends Readonly<{ [K in TActionData["id"]]: ActionBuilderWithFunction<TActionData>; }>, U extends (keyof TRegistry)[] | undefined = undefined>(registry: TRegistry, args: TFunctionCallingArgs<TRegistry, U>) => {
    /**
     * A function that calls the actions.
     *
     * Returns the results of the actions. Results are in the same order as the inputs and contain the status of the action, the data if the action was successful, and the action ID.
     *
     * @param inputs The inputs to the actions.
     * @returns The results of the actions.
     */
    actionCaller: TActionCaller<TActionData, TRegistry, U>;
};

type TActionsArray<TRegistry extends TAnyRegistryData> = Array<ReturnType<ActionBuilderWithFunction<any>["getRegistryData"]> extends TRegistry ? ActionBuilderWithFunction<any> : never>;
type TActionsRegistry<TRegistry extends TAnyRegistryData, T extends TActionsArray<TRegistry>> = T extends [infer A, ...infer R] ? A extends ActionBuilderWithFunction<any> ? ReturnType<A["getRegistryData"]> extends infer ARegistryData ? ARegistryData extends TRegistry ? ReturnType<A["getId"]> extends infer TId ? TId extends string ? R extends TActionsArray<TRegistry> ? Readonly<{
    [K in TId]: A;
}> & TActionsRegistry<TRegistry, R> : never : never : never : never : never : never : Readonly<{}>;
/**
 * Filter the action registry by the auth type.
 *
 * @param registry The registry of actions.
 * @param authType The auth type to filter by.
 *
 * @returns The filtered registry of actions.
 */
declare const filterActionRegistryByAuthType: <const T extends Readonly<{
    [key: string]: ActionBuilderWithFunction<any>;
}>, U extends TAuthType>(registry: T, authType: U) => { [K in keyof T as ReturnType<T[K]["getAuthConfig"]>["type"] extends U ? K : never]: T[K]; };
declare const generateActionIdMap: <const T extends Readonly<{
    [key: string]: ActionBuilderWithFunction<any>;
}>>(registry: T) => Readonly<{ [K in keyof T]: K; }>;
/**
 * Filter the action registry by the action type.
 *
 * @param registry The registry of actions.
 * @param authType The auth type to filter by.
 *
 * @returns The filtered registry of actions.
 */
declare const filterActionRegistryByActionType: <const T extends Readonly<{
    [key: string]: ActionBuilderWithFunction<any>;
}>, U extends TActionType>(registry: T, actionType: U) => { [K in keyof T as U extends "POST" ? ReturnType<T[K]["getOutputSchema"]> extends z__default.ZodVoid ? K : never : ReturnType<T[K]["getOutputSchema"]> extends z__default.AnyZodObject ? K : never]: T[K]; };
type TCreateActionsRegistry<T extends string> = `create${T}ActionsRegistry`;
type TCreateAction<T extends string> = `create${T}Action`;
type TCreateActionsRegistryFunction<TRegistry extends TAnyRegistryData> = <const T extends TActionsArray<TRegistry>>(registry: T) => TActionsRegistry<TRegistry, T>;
type TCreateActionFunction<TRegistry extends TAnyRegistryData> = TRegistry["metadataSchema"] extends infer TMetadataSchema ? TMetadataSchema extends TActionMetadata | unknown ? <TId extends string>(input: {
    id: TId;
} & (TMetadataSchema extends z__default.AnyZodObject ? {
    metadata: z__default.input<TMetadataSchema>;
} : {})) => ActionBuilder<TActionBuilderConstructorData<TRegistry, TId>> : never : never;
type TGenerateFunctionsRet<TRegistry extends TAnyRegistryData> = TRegistry["namespace"] extends infer TNamespace ? TNamespace extends string ? Readonly<{
    [K in TCreateActionsRegistry<TNamespace>]: TCreateActionsRegistryFunction<TRegistry>;
} & {
    [K in TCreateAction<TNamespace>]: TCreateActionFunction<TRegistry>;
}> : never : never;
/**
 * Generate the functions to create an action registry.
 *
 * @param args The arguments to generate the functions.
 * @returns The functions to create an action registry.
 *
 * @example
 * ```
 * const { createADEActionsRegistry, createADEAction } = generateActionRegistryFunctions({
 *   namespace: "ADE",
 *   metadataSchema: z.object({
 *     title: z.string(),
 *     description: z.string(),
 *   }),
 *   actionFunctionExtras: z.object({
 *     userData: z.object({
 *       email: z.string().email(),
 *       name: z.string(),
 *     })
 *   }),
 * })
 * ```
 */
declare const generateActionRegistryFunctions: <const TRegistry extends TAnyRegistryData>(args: TRegistry) => TGenerateFunctionsRet<TRegistry>;
declare const createActionsRegistry: TCreateActionsRegistryFunction<{
    readonly namespace: "";
    readonly metadataSchema: undefined;
    readonly actionFunctionExtrasSchema: undefined;
    readonly tokenAuthMetadataSchema: undefined;
    readonly oAuthMetadataSchema: undefined;
}>;
declare const createAction: <TId extends string>(input: {
    id: TId;
}) => ActionBuilder<TActionBuilderConstructorData<{
    readonly namespace: "";
    readonly metadataSchema: undefined;
    readonly actionFunctionExtrasSchema: undefined;
    readonly tokenAuthMetadataSchema: undefined;
    readonly oAuthMetadataSchema: undefined;
}, TId>>;
type inferMetadataSchema<T> = T extends TCreateActionFunction<infer TRegistry> ? TRegistry["metadataSchema"] extends infer TMetadata ? TMetadata extends TActionMetadata ? TMetadata : never : never : never;
type inferActionComponent<T, TInput extends TActionInput, TOnSubmitValues extends TActionOnSubmit = undefined> = TActionComponent<inferMetadataSchema<T>, TInput, TOnSubmitValues>;

/**
 * Given an action registry, generate the tools for the LLM.
 *
 * @param registry The registry of actions.
 * @param toolId The id(s) of the action(s) to generate tools for. Can be a single id or an array of ids.
 *
 * @returns The tools for the LLM. In the format of `OpenAI.Chat.Completions.ChatCompletionTool[]`. The name of the tool is the ID of the action.
 *
 * @example
 */
declare const generateLLMTools: <const T extends Readonly<{
    [key: string]: ActionBuilderWithFunction<any>;
}>, U extends (keyof T)[] | undefined>(registry: T, args?: {
    inArray?: U | undefined;
} | undefined) => OpenAI.Chat.Completions.ChatCompletionTool[];
type TToolCallHandler<T extends Readonly<{
    [key: string]: ActionBuilderWithFunction<any>;
}>, U extends (keyof T)[] | undefined> = (toolCalls: OpenAI.Chat.Completions.ChatCompletionMessageToolCall[]) => Promise<{
    toolCallMessages: OpenAI.Chat.Completions.ChatCompletionToolMessageParam[];
    results: TCallerResults<T, TActionRegistrySubset<T, U>>;
}>;
declare const setupFunctionCalling: <TActionData extends (TActionBuilderData & Pick<TActionData<any, any, TActionInput, any, any, any>, "inputSchema"> & Pick<TActionData<TAnyRegistryData, string, TActionInput, TActionOutput, any, TActionOnSubmit>, "submissionSchema" | "outputSchema" | "component"> & {
    authConfig: TAnyActionAuth;
} & Pick<TActionData<TAnyRegistryData, string, TActionInput, TActionOutput, {
    type: "None";
    config: undefined;
}, TActionOnSubmit>, "actionFunction" | "exampleInput">) | (TActionBuilderData & Pick<TActionData<any, any, TActionInput, any, any, any>, "inputSchema"> & Pick<TActionData<TAnyRegistryData, string, TActionInput, TActionOutput, any, TActionOnSubmit>, "submissionSchema" | "outputSchema" | "component"> & {
    authConfig: TAnyActionAuth;
} & Pick<TActionData<TAnyRegistryData, string, TActionInput, TActionOutput, TTokenAuth<TTokenCustomData, TAnyRegistryData>, TActionOnSubmit>, "actionFunction" | "exampleInput">) | (TActionBuilderData & Pick<TActionData<any, any, TActionInput, any, any, any>, "inputSchema"> & Pick<TActionData<TAnyRegistryData, string, TActionInput, TActionOutput, any, TActionOnSubmit>, "submissionSchema" | "outputSchema" | "component"> & {
    authConfig: TAnyActionAuth;
} & Pick<TActionData<TAnyRegistryData, string, TActionInput, TActionOutput, {
    type: "OAuth";
    config: TOAuthConfigWithOutputMetadata<TAnyRegistryData>;
}, TActionOnSubmit>, "actionFunction" | "exampleInput">), const T extends Readonly<{ [K in TActionData["id"]]: ActionBuilderWithFunction<TActionData>; }>, U extends (keyof T)[] | undefined = undefined>(registry: T, args: {
    inArray?: U | undefined;
    runInParallel?: boolean | undefined;
} & (ReturnType<ValuesOf<T>["getRegistryData"]> extends infer TRegistry ? TRegistry extends TAnyRegistryData ? TRegistry["actionFunctionExtrasSchema"] extends infer A ? A extends z.AnyZodObject ? {
    extras: z.input<A>;
} : {} : {} : {} : {}) & ((TActionRegistrySubset<T, U> extends infer T_1 extends keyof T ? { [K_1 in T_1 as ReturnType<T[K_1]["getAuthConfig"]>["type"] extends "OAuth" ? K_1 : never]: 1; } : never) extends infer A_1 ? keyof A_1 extends never ? {} : {
    fetchOAuthAccessToken: (actionId: keyof A_1) => Promise<{
        accessToken: string;
    }>;
} : {}) & ((TActionRegistrySubset<T, U> extends infer T_2 extends keyof T ? { [K_2 in T_2 as ReturnType<T[K_2]["getAuthConfig"]>["type"] extends "Token" ? K_2 : never]: 1; } : never) extends infer A_2 ? keyof A_2 extends never ? {} : {
    fetchTokenAuthData: (actionId: keyof A_2) => Promise<{
        accessToken: string;
        customData: {
            [x: string]: any;
        };
    }>;
} : {}) & {
    /**
     * Whether to include errors in the tool call messages.
     */
    includeErrorsInToolCallMessages?: boolean | undefined;
}) => {
    tools: OpenAI.Chat.Completions.ChatCompletionTool[];
    toolCallsHandler: TToolCallHandler<T, U>;
};

export { AuthType, type FilterObjectByValue, type TActionCallerInput, type TActionRegistrySubset, type TCallerErrorResult, type TCallerResults, type TCallerSuccessResult, type TFunctionCallingArgs, type ValuesOf, createAction, createActionsRegistry, filterActionRegistryByActionType, filterActionRegistryByAuthType, generateActionIdMap, generateActionRegistryFunctions, generateLLMTools, type inferActionComponent, setupActionCaller, setupFunctionCalling };
